@@ -32,7 +32,7 @@ sys.path.append(os.path.join(dir_, "ext"))
 extensions = ['sphinx.ext.autodoc', 'sphinx.ext.extlinks', 'contributors']
 
 source_suffix = '.rst'
-master_doc = 'index'
+root_doc = 'index'
 project = 'Quod Libet'
 copyright = u""
 exclude_patterns = ['_build', '_build_all', 'README.rst', '**/README.rst']
@@ -40,9 +40,9 @@ html_theme = "sphinx_rtd_theme"
 html_title = project
 
 extlinks = {
-    'bug': ('https://github.com/quodlibet/quodlibet/issues/%s', '#'),
-    'pr': ('https://github.com/quodlibet/quodlibet/pull/%s', '#'),
-    'user': ('https://github.com/%s', ''),
+    'bug': ('https://github.com/quodlibet/quodlibet/issues/%s', '#%s'),
+    'pr': ('https://github.com/quodlibet/quodlibet/pull/%s', '#%s'),
+    'user': ('https://github.com/%s', '%s'),
 }
 
 linkcheck_anchors = True
