@@ -25,6 +25,7 @@ import os
 import sys
 import subprocess
 import tarfile
+import fnmatch
 from distutils import dir_util
 
 from .util import Command, get_dist_class
@@ -77,7 +78,7 @@ class test_cmd(Command):
 
 
 class quality_cmd(Command):
-    description = "Run pep8/pyflakes tests"
+    description = "Run flake8/mypy tests"
     user_options = []
 
     def initialize_options(self):
@@ -118,6 +119,17 @@ class distcheck_cmd(sdist):
             assert process.returncode == 0
 
             tracked_files = out.splitlines()
+            ignore_tracked = [
+                "dev-utils/*",
+                ".github/*",
+                ".ci/*",
+                ".codecov.yml",
+                ".editorconfig",
+                ".git*",
+            ]
+            tracked_files = [
+                p for p in tracked_files if not
+                any(fnmatch.fnmatch(p, i) for i in ignore_tracked)]
 
             diff = set(tracked_files) - set(included_files)
             assert not diff, (
