@@ -1,23 +1,25 @@
-# -*- coding: utf-8 -*-
 # Copyright 2013 Christoph Reiter
+#        2020-22 Nick Boultbee
 #
 # This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License version 2 as
-# published by the Free Software Foundation
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
 
 import re
+from typing import Tuple, Text
+from quodlibet.browsers.paned.util import PaneConfig
 
 from quodlibet import _
 from quodlibet import util
 from quodlibet.qltk.models import ObjectStore
 from quodlibet.util.collection import Collection
-from quodlibet.compat import iteritems, listfilter
 
 
 class BaseEntry(Collection):
 
     def __init__(self, key=None, songs=None):
-        super(BaseEntry, self).__init__()
+        super().__init__()
 
         self.songs = set(songs or [])
         self.key = key # not used for sorting!
@@ -53,14 +55,14 @@ class BaseEntry(Collection):
 class SongsEntry(BaseEntry):
 
     def __init__(self, key, sort, songs=None):
-        super(SongsEntry, self).__init__(key, songs)
+        super().__init__(key, songs)
 
         self.sort = sort # value used for sorting
 
     def get_count_text(self, config):
         return config.format_display(self)
 
-    def get_text(self, config):
+    def get_text(self, config: PaneConfig) -> Tuple[bool, Text]:
         if config.has_markup:
             return True, self.key
         else:
@@ -80,10 +82,10 @@ class SongsEntry(BaseEntry):
 class UnknownEntry(SongsEntry):
 
     def __init__(self, songs=None):
-        super(UnknownEntry, self).__init__("", tuple(), songs)
+        super().__init__("", tuple(), songs)
 
     def get_text(self, config):
-        return True, "<b>%s</b>" % _("Unknown")
+        return True, util.bold(_("Unknown"))
 
     def contains_text(self, text):
         return False
@@ -95,13 +97,13 @@ class UnknownEntry(SongsEntry):
 class AllEntry(BaseEntry):
 
     def __init__(self):
-        super(AllEntry, self).__init__()
+        super().__init__()
 
     def get_count_text(self, config):
         return u""
 
     def get_text(self, config):
-        return True, "<b>%s</b>" % _("All")
+        return True, util.bold(_("All"))
 
     def contains_text(self, text):
         return False
@@ -116,7 +118,7 @@ class AllEntry(BaseEntry):
 class PaneModel(ObjectStore):
 
     def __init__(self, pattern_config):
-        super(PaneModel, self).__init__()
+        super().__init__()
         self.__sort_cache = {} # text to sort text cache
         self.__key_cache = {} # song to key cache
         self.config = pattern_config
@@ -126,8 +128,7 @@ class PaneModel(ObjectStore):
             return self.__key_cache[song]
         except KeyError:
             # We filter out empty values, so Unknown can be ""
-            self.__key_cache[song] = listfilter(
-                lambda v: v[0], self.config.format(song))
+            self.__key_cache[song] = [v for v in self.config.format(song) if v[0]]
             return self.__key_cache[song]
 
     def __human_sort_key(self, text, reg=re.compile('<.*?>')):
@@ -226,7 +227,7 @@ class PaneModel(ObjectStore):
                     collection[key] = (entry, hsort, bool(sort))
                     entry.songs.add(song)
 
-        items = sorted(iteritems(collection),
+        items = sorted(collection.items(),
                        key=lambda s: s[1][1],
                        reverse=True)
 
