@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Quod Libet Telepathy Plugin
 # Copyright 2012 Christoph Reiter
 #      2012,2016 Nick Boultbee
@@ -129,8 +128,7 @@ class TelepathyStatusPlugin(EventPlugin, PluginConfigMixin):
         lbl = Gtk.Label(label=_("Playing:"))
         entry.set_tooltip_markup(_("Status text when a song is started. "
                                  "Accepts QL Patterns e.g. %s")
-                                 % util.monospace(
-                                        util.escape("<~artist~title>")))
+                                 % util.monospace("<~artist~title>"))
         lbl.set_mnemonic_widget(entry)
         hb.pack_start(lbl, False, True, 0)
         hb.pack_start(entry, True, True, 0)
@@ -146,8 +144,7 @@ class TelepathyStatusPlugin(EventPlugin, PluginConfigMixin):
         lbl = Gtk.Label(label=_("Paused:"))
         entry.set_tooltip_markup(_("Status text when a song is paused. "
                                    "Accepts QL Patterns e.g. %s")
-                                   % util.monospace(
-                                        util.escape("<~artist~title>")))
+                                   % util.monospace("<~artist~title>"))
         lbl.set_mnemonic_widget(entry)
         hb.pack_start(lbl, False, True, 0)
         hb.pack_start(entry, True, True, 0)
