@@ -1,18 +1,18 @@
-# -*- coding: utf-8 -*-
 # Copyright 2004-2005 Joe Wreschnig, Michael Urman
 #
 # This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License version 2 as
-# published by the Free Software Foundation
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+from typing import List
 
 from senf import fsnative, path2fsn
 
-from quodlibet.compat import text_type
-
 from ._audio import AudioFile
 
 
-extensions = []
+extensions: List[str] = []
 
 
 class RemoteFile(AudioFile):
@@ -22,12 +22,13 @@ class RemoteFile(AudioFile):
     format = "Remote File"
 
     def __init__(self, uri):
-        self["~uri"] = text_type(uri)
+        assert not isinstance(uri, bytes)
+        self["~uri"] = str(uri)
         self.sanitize(fsnative(self["~uri"]))
 
     def __getitem__(self, key):
         # we used to save them with the wrong type
-        value = super(RemoteFile, self).__getitem__(key)
+        value = super().__getitem__(key)
         if key in ("~filename", "~mountpoint") and \
                 not isinstance(value, fsnative):
             value = path2fsn(value)
