@@ -1,9 +1,9 @@
-# -*- coding: utf-8 -*-
 # Copyright 2012,2013 Christoph Reiter
 #
 # This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License version 2 as
-# published by the Free Software Foundation
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
 
 # TODO:
 # RenameCommand
@@ -24,7 +24,6 @@ from quodlibet.util.path import mtime
 from quodlibet.pattern import Pattern, error as PatternError
 from quodlibet.util.tags import USER_TAGS, sortkey, MACHINE_TAGS
 from quodlibet.util.tagsfrompath import TagsFromPattern
-from quodlibet.compat import text_type, iteritems
 
 from .base import Command, CommandError
 from .util import print_terse_table, copy_mtime, list_tags, print_table, \
@@ -142,8 +141,9 @@ class CopyCommand(Command):
         for key in source.realkeys():
             self.log("Copy %r" % key)
             if not options.ignore_errors and not dest.can_change(key):
-                raise CommandError(_("Can't copy tag %r to file: %r") %
-                                   (key, dest_path))
+                raise CommandError(
+                    _("Can't copy tag {tagname} to file: {filename}").format(
+                        tagname=repr(key), filename=repr(dest_path)))
             for value in source.list(key):
                 dest.add(key, value)
 
@@ -180,7 +180,7 @@ class EditCommand(Command):
         return u"\n".join(lines)
 
     def _text_to_song(self, text, song):
-        assert isinstance(text, text_type)
+        assert isinstance(text, str)
 
         # parse
         tags = {}
@@ -207,7 +207,7 @@ class EditCommand(Command):
                     self.log("Add %s=%s" % (key, value))
                     song.add(key, value)
 
-        for key, values in iteritems(tags):
+        for key, values in tags.items():
             if not song.can_change(key):
                 raise CommandError(
                     "Can't change key '%(key-name)s'." % {"key-name": key})
@@ -227,16 +227,16 @@ class EditCommand(Command):
         # write to tmp file
         fd, path = tempfile.mkstemp(suffix=".txt")
 
-        # XXX: copy mtime here so we can test for changes in tests by
-        # setting a out of date mtime on the source song file
-        copy_mtime(args[0], path)
-
         try:
             try:
                 os.write(fd, dump)
             finally:
                 os.close(fd)
 
+            # XXX: copy mtime here so we can test for changes in tests by
+            # setting a out of date mtime on the source song file
+            copy_mtime(args[0], path)
+
             # only parse the result if the editor returns 0 and the mtime has
             # changed
             old_mtime = mtime(path)
@@ -247,10 +247,10 @@ class EditCommand(Command):
             try:
                 subprocess.check_call(editor_args + [path])
             except subprocess.CalledProcessError as e:
-                self.log(text_type(e))
+                self.log(str(e))
                 raise CommandError(_("Editing aborted"))
             except OSError as e:
-                self.log(text_type(e))
+                self.log(str(e))
                 raise CommandError(
                     _("Starting text editor '%(editor-name)s' failed.") % {
                         "editor-name": editor_args[0]})
@@ -301,7 +301,9 @@ class SetCommand(Command):
             song = self.load_song(path)
 
             if not song.can_change(tag):
-                raise CommandError(_("Can not set %r") % tag)
+                vars = dict(tag=tag, format=type(song).format, file=song("~filename"))
+                raise CommandError(
+                    _("Can not set %(tag)r for %(format)s file %(file)r") % vars)
 
             self.log("Set %r to %r" % (value, tag))
             if tag in song:
@@ -363,7 +365,8 @@ class ClearCommand(Command):
                 self.log("Remove tag %r" % tag)
                 if not song.can_change(tag):
                     raise CommandError(
-                        _("Can't remove %r from %r") % (tag, path))
+                        _("Can't remove {tagname} from {filename}").format(
+                            tagname=repr(tag), filename=repr(path)))
                 del song[tag]
 
             if tags:
@@ -484,14 +487,14 @@ class InfoCommand(Command):
             tags = []
             for key in ["~format", "~codec", "~encoding", "~length",
                         "~bitrate", "~filesize"]:
-                tags.append((util.tag(key), text_type(song.comma(key))))
+                tags.append((util.tag(key), str(song.comma(key))))
 
             print_table(tags, headers, nicks, order)
         else:
             tags = []
             for key in ["~format", "~codec", "~encoding", "~#length",
                         "~#bitrate", "~#filesize"]:
-                tags.append((key.lstrip("#~"), text_type(song(key))))
+                tags.append((key.lstrip("#~"), str(song(key))))
 
             print_terse_table(tags, nicks, order)
 
