@@ -1,9 +1,10 @@
-# -*- coding: utf-8 -*-
 # Copyright 2004 Joe Wreschnig, Michael Urman
+#           2021 Nick Boultbee
 #
 # This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License version 2 as
-# published by the Free Software Foundation
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
 
 from quodlibet.player._base import BasePlayer
 from quodlibet.player import PlayerError
@@ -14,7 +15,7 @@ class NullPlayer(BasePlayer):
     name = "Null"
 
     def __init__(self, sinkname="", librarian=None):
-        super(NullPlayer, self).__init__()
+        super().__init__()
         self._paused = True
         self._source = None
         self._volume = 1.0
@@ -66,9 +67,10 @@ class NullPlayer(BasePlayer):
 
     def seek(self, pos):
         """Seek to a position in the song, in milliseconds."""
+
+        self._position = pos
         if self.song:
             self.emit('seek', self.song, pos)
-        self._position = pos
 
     def _end(self, stopped, next_song=None):
         # We need to set self.song to None before calling our signal
@@ -78,9 +80,10 @@ class NullPlayer(BasePlayer):
         self.song = self.info = None
         self.emit('song-ended', song, stopped)
 
-        current = self._source.current if next_song is None else next_song
+        current = next_song if next_song else (self._source and self._source.current)
 
         # Then, set up the next song.
+        self._position = 0
         self.song = self.info = current
         self.emit('song-started', self.song)
 
@@ -90,7 +93,6 @@ class NullPlayer(BasePlayer):
         if not self.paused and song is None:
             self.emit("unpaused")
 
-        self._position = 0
         # seekable might change if we change to None, so notify just in case
         self.notify("seekable")
 
