@@ -6,6 +6,8 @@
 # the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
 
+from typing import Set
+
 from gi.repository import Gtk
 
 from quodlibet import formats, config, print_d
@@ -23,7 +25,7 @@ class EntryWordCompletion(Gtk.EntryCompletion):
     rightsep = [" ", ")", ","]
 
     def __init__(self):
-        super(EntryWordCompletion, self).__init__()
+        super().__init__()
         self.set_match_func(self.__match_filter, None)
         self.connect('match-selected', self.__match_selected)
 
@@ -74,10 +76,10 @@ class EntryWordCompletion(Gtk.EntryCompletion):
 class LibraryTagCompletion(EntryWordCompletion):
     """A completion for text entries tied to a library's tag list."""
 
-    __tags = set()
+    __tags: Set[str] = set()
 
     def __init__(self, library):
-        super(LibraryTagCompletion, self).__init__()
+        super().__init__()
         try:
             model = self.__model
         except AttributeError:
@@ -91,7 +93,8 @@ class LibraryTagCompletion(EntryWordCompletion):
 
     @classmethod
     def __update_song(klass, library, songs, model):
-        print_d("Updating tag model for %d songs" % len(songs))
+        if not songs:
+            return
         tags = klass.__tags
         for song in songs:
             for tag in song.keys():
@@ -99,7 +102,7 @@ class LibraryTagCompletion(EntryWordCompletion):
                         or tag in tags):
                     klass.__tags.add(tag)
                     model.append([tag])
-        print_d("Done updating tag model for %d songs" % len(songs))
+        print_d("Updated tag model for %d songs" % len(songs))
 
     @classmethod
     def __build_model(klass, library, model):
@@ -143,7 +146,7 @@ class LibraryValueCompletion(Gtk.EntryCompletion):
     Will add valid values from the tag massager where available"""
 
     def __init__(self, tag, library):
-        super(LibraryValueCompletion, self).__init__()
+        super().__init__()
         self.set_model(Gtk.ListStore(str))
         self.set_text_column(0)
         self.set_tag(tag, library)
