@@ -1,5 +1,5 @@
 # Copyright 2004-2012 Joe Wreschnig, Michael Urman, Iñigo Serna
-#           2011-2017 Nick Boultbee
+#           2011-2022 Nick Boultbee
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -7,38 +7,38 @@
 # (at your option) any later version.
 
 import sys
+from typing import Optional, Type, Sequence
 
 from gi.repository import Gtk, Pango, Gdk
 
-from quodlibet import C_, _
-from quodlibet import qltk
-from quodlibet import config
-from quodlibet import util
+from quodlibet import C_, _, ngettext, print_e, print_d
 from quodlibet import app
-
-from quodlibet.util import massagers
-
+from quodlibet import config
+from quodlibet import qltk
+from quodlibet import util
 from quodlibet.formats import AudioFileError
-from quodlibet.qltk.completion import LibraryValueCompletion
-from quodlibet.qltk.tagscombobox import TagsComboBox, TagsComboBoxEntry
-from quodlibet.qltk.views import RCMHintedTreeView, TreeViewColumn
-from quodlibet.qltk.wlw import WritingWindow
-from quodlibet.qltk.window import Dialog
-from quodlibet.qltk.models import ObjectStore
-from quodlibet.qltk.ccb import ConfigCheckButton
-from quodlibet.qltk.x import SeparatorMenuItem, Button, MenuItem
+from quodlibet.plugins import PluginManager
+from quodlibet.plugins.editing import EditTagsPlugin
+from quodlibet.qltk import Icons
 from quodlibet.qltk._editutils import EditingPluginHandler, OverwriteWarning
 from quodlibet.qltk._editutils import WriteFailedError
-from quodlibet.qltk import Icons
-from quodlibet.plugins import PluginManager
+from quodlibet.qltk.ccb import ConfigCheckButton
+from quodlibet.qltk.completion import LibraryValueCompletion
+from quodlibet.qltk.models import ObjectStore
+from quodlibet.qltk.tagscombobox import TagsComboBox, TagsComboBoxEntry
+from quodlibet.qltk.views import RCMHintedTreeView, TreeViewColumn, BaseView
+from quodlibet.qltk.window import Dialog
+from quodlibet.qltk.wlw import WritingWindow
+from quodlibet.qltk.x import SeparatorMenuItem, Button, MenuItem
 from quodlibet.util import connect_obj
+from quodlibet.util import massagers
 from quodlibet.util.i18n import numeric_phrase
-from quodlibet.util.tags import USER_TAGS, MACHINE_TAGS, sortkey as tagsortkey
 from quodlibet.util.string.splitters import (split_value, split_title,
-    split_people, split_album)
+                                             split_people, split_album)
+from quodlibet.util.tags import USER_TAGS, MACHINE_TAGS, sortkey as tagsortkey
 
 
-class Comment(object):
+class Comment:
     """A summary of a collection of values for one tag"""
 
     def __init__(self, text, have=1, total=1, shared=True):
@@ -93,11 +93,10 @@ class Comment(object):
             return util.escape(self.text)
         elif self.shared:
             return "\n".join(
-                ['%s<i> (%s)</i>' % (util.escape(s),
-                                     util.escape(self._paren()))
+                [f"{util.escape(s)}{util.italic(' ' + self._paren())}"
                  for s in self.text.split("\n")])
         else:
-            return '<i>(%s)</i>' % util.escape(self._paren())
+            return util.italic(self._paren())
 
 
 def get_default_tags():
@@ -187,13 +186,15 @@ class AudioFileGroup(dict):
         return all(song.can_change(key) for song in self.songs)
 
 
-class SplitValues(Gtk.ImageMenuItem):
-    tags = False
-    needs = []
-    _order = 0.0
+class TagSplitter(EditTagsPlugin):
+    """Splits tag values into other tags"""
+    pass
+
+
+class SplitValues(TagSplitter):
 
     def __init__(self, tag, value):
-        super(SplitValues, self).__init__(
+        super().__init__(
             label=_("Split into _Multiple Values"), use_underline=True)
         self.set_image(Gtk.Image.new_from_icon_name(
             Icons.EDIT_FIND_REPLACE, Gtk.IconSize.MENU))
@@ -209,13 +210,13 @@ class SplitValues(Gtk.ImageMenuItem):
         return [(tag, v) for v in split_value(value, spls)]
 
 
-class SplitDisc(Gtk.ImageMenuItem):
+class SplitDisc(TagSplitter):
     tags = ["album"]
     needs = ["discnumber"]
     _order = 0.5
 
     def __init__(self, tag, value):
-        super(SplitDisc, self).__init__(
+        super().__init__(
             label=_("Split Disc out of _Album"), use_underline=True)
         self.set_image(Gtk.Image.new_from_icon_name(
             Icons.EDIT_FIND_REPLACE, Gtk.IconSize.MENU))
@@ -224,7 +225,7 @@ class SplitDisc(Gtk.ImageMenuItem):
         if disc is not None:
             album = album if len(album) <= 64 else album[:64] + "…"
             self.set_label("{}={}, {}={}".format(tag, album,
-                           self.needs[0], disc))
+                                                 self.needs[0], disc))
 
         self.set_sensitive(disc is not None)
 
@@ -233,13 +234,13 @@ class SplitDisc(Gtk.ImageMenuItem):
         return [(tag, album), ("discnumber", disc)]
 
 
-class SplitTitle(Gtk.ImageMenuItem):
+class SplitTitle(TagSplitter):
     tags = ["title"]
     needs = ["version"]
     _order = 0.5
 
     def __init__(self, tag, value):
-        super(SplitTitle, self).__init__(
+        super().__init__(
             label=_("Split _Version out of Title"), use_underline=True)
         self.set_image(Gtk.Image.new_from_icon_name(
             Icons.EDIT_FIND_REPLACE, Gtk.IconSize.MENU))
@@ -252,7 +253,8 @@ class SplitTitle(Gtk.ImageMenuItem):
             versions = [ver if len(ver) <= 64 else ver[:64] + "…"
                         for ver in versions]
             string = (", ".join(["{}={}".format(tag, title)] +
-               ["{}={}".format(self.needs[0], ver) for ver in versions]))
+                                ["{}={}".format(self.needs[0], ver) for ver in
+                                 versions]))
             self.set_label(string)
 
         self.set_sensitive(bool(versions))
@@ -264,12 +266,12 @@ class SplitTitle(Gtk.ImageMenuItem):
         return [(tag, title)] + [("version", v) for v in versions]
 
 
-class SplitPerson(Gtk.ImageMenuItem):
+class SplitPerson(TagSplitter):
     tags = ["artist"]
     _order = 0.5
 
     def __init__(self, tag, value):
-        super(SplitPerson, self).__init__(label=self.title, use_underline=True)
+        super().__init__(label=self.title, use_underline=True)
         self.set_image(Gtk.Image.new_from_icon_name(
             Icons.EDIT_FIND_REPLACE, Gtk.IconSize.MENU))
         tag_spls = config.gettext("editing", "split_on").split()
@@ -279,9 +281,10 @@ class SplitPerson(Gtk.ImageMenuItem):
         if others:
             artist = artist if len(artist) <= 64 else artist[:64] + "…"
             others = [other if len(other) <= 64 else other[:64] + "…"
-                        for other in others]
+                      for other in others]
             string = (", ".join(["{}={}".format(tag, artist)] +
-                ["{}={}".format(self.needs[0], o) for o in others]))
+                                ["{}={}".format(self.needs[0], o) for o in
+                                 others]))
             self.set_label(string)
 
         self.set_sensitive(bool(others))
@@ -318,7 +321,7 @@ class SplitOriginalArtistFromTitle(SplitPerson):
 class AddTagDialog(Dialog):
 
     def __init__(self, parent, can_change, library):
-        super(AddTagDialog, self).__init__(
+        super().__init__(
             title=_("Add a Tag"), transient_for=qltk.get_top_parent(parent),
             use_header_bar=True)
         self.set_border_width(6)
@@ -406,7 +409,7 @@ class AddTagDialog(Dialog):
         self.show()
         self.__val.set_activates_default(True)
         self.__tag.grab_focus()
-        return super(AddTagDialog, self).run()
+        return super().run()
 
 
 class EditTagsPluginHandler(EditingPluginHandler):
@@ -414,7 +417,7 @@ class EditTagsPluginHandler(EditingPluginHandler):
     Kind = EditTagsPlugin
 
 
-class ListEntry(object):
+class ListEntry:
     """Holds a Comment and some state for the editing process"""
 
     tag = None
@@ -434,14 +437,20 @@ class ListEntry(object):
 class EditTags(Gtk.VBox):
     handler = EditTagsPluginHandler()
 
+    _SPLITTERS: Sequence[Type[TagSplitter]] = sorted(
+        [SplitDisc, SplitTitle, SplitPerformer, SplitArranger, SplitValues,
+         SplitPerformerFromTitle, SplitOriginalArtistFromTitle],
+        key=lambda item: (item._order, item.__name__))
+
     @classmethod
     def init_plugins(cls):
         PluginManager.instance.register_handler(cls.handler)
 
     def __init__(self, parent, library):
-        super(EditTags, self).__init__(spacing=12)
+        super().__init__(spacing=12)
         self.title = _("Edit Tags")
         self.set_border_width(12)
+        self._group_info = None
 
         model = ObjectStore()
         view = RCMHintedTreeView(model=model)
@@ -463,6 +472,7 @@ class EditTags(Gtk.VBox):
                     rend.set_property('icon-name', Icons.EDIT)
             else:
                 rend.set_property('icon-name', Icons.CHANGES_PREVENT)
+
         column.set_cell_data_func(render, cdf_write)
         view.append_column(column)
 
@@ -516,15 +526,26 @@ class EditTags(Gtk.VBox):
             _("Show _programmatic tags"), 'editing', 'alltags', populate=True,
             tooltip=_("Access all tags, including machine-generated "
                       "ones e.g. MusicBrainz or Replay Gain tags"))
-        cb.connect('toggled', self.__all_tags_toggled)
-        self.pack_start(cb, False, True, 0)
+        cb.connect('toggled', self.__checkbox_toggled)
+        hb = Gtk.HBox()
+        hb.pack_start(cb, False, True, 0)
+
+        cb = ConfigCheckButton(
+            _("Show _multi-line tags"),
+            "editing",
+            "show_multi_line_tags",
+            populate=True,
+            tooltip=_("Show potentially multi-line tags (e.g 'lyrics') here too"))
+        cb.connect('toggled', self.__checkbox_toggled)
+        hb.pack_start(cb, False, True, 12)
+        self.pack_start(hb, False, True, 0)
 
         # Add and Remove [tags] buttons
         buttonbox = Gtk.HBox(spacing=18)
         bbox1 = Gtk.HButtonBox()
         bbox1.set_spacing(6)
         bbox1.set_layout(Gtk.ButtonBoxStyle.START)
-        add = qltk.Button(_("_Add"), Icons.LIST_ADD)
+        add = qltk.Button(_("_Add…"), Icons.LIST_ADD)
         add.set_focus_on_click(False)
         self._add = add
         add.connect('clicked', self.__add_tag, model, library)
@@ -570,7 +591,7 @@ class EditTags(Gtk.VBox):
             model.connect(sig, self.__enable_save, [save, revert])
             connect_obj(model, sig, parent.set_pending, save)
 
-        view.connect('popup-menu', self.__popup_menu, parent)
+        view.connect('popup-menu', self._popup_menu, parent)
         view.connect('button-press-event', self.__button_press)
         view.connect('key-press-event', self.__view_key_press_event)
         selection.connect('changed', self.__tag_select, remove)
@@ -581,7 +602,7 @@ class EditTags(Gtk.VBox):
         for child in self.get_children():
             child.show_all()
 
-    def __all_tags_toggled(self, *args):
+    def __checkbox_toggled(self, *args):
         self._update()
 
     def __view_key_press_event(self, view, event):
@@ -632,18 +653,25 @@ class EditTags(Gtk.VBox):
             entry.edited = entry.deleted = True
             model.path_changed(path)
 
-    def __popup_menu(self, view, parent):
+    def __item_for(self, view: BaseView,
+                   Item: Type[EditTagsPlugin],
+                   tag: str,
+                   text: str) -> Optional[EditTagsPlugin]:
+        try:
+            item = Item(tag, text)
+        except Exception as e:
+            print_e(f"Couldn't create menu item from {Item} ({e})")
+            return None
+        else:
+            item.connect('activate', self.__menu_activate, view)
+            return item
+
+    def _popup_menu(self, view: BaseView, _parent):
         menu = Gtk.Menu()
 
         view.ensure_popup_selection()
         model, rows = view.get_selection().get_selected_rows()
-        can_change = min([model[path][0].canedit for path in rows])
-
-        items = [SplitDisc, SplitTitle, SplitPerformer, SplitArranger,
-                 SplitValues, SplitPerformerFromTitle,
-                 SplitOriginalArtistFromTitle]
-        items.extend(self.handler.plugins)
-        items.sort(key=lambda item: (item._order, item.__name__))
+        can_change = all(model[path][0].canedit for path in rows)
 
         if len(rows) == 1:
             row = model[rows[0]]
@@ -654,30 +682,34 @@ class EditTags(Gtk.VBox):
 
             split_menu = Gtk.Menu()
 
-            for Item in items:
+            for Item in self._SPLITTERS:
                 if Item.tags and entry.tag not in Item.tags:
                     continue
-
-                try:
-                    b = Item(entry.tag, text)
-                except:
-                    util.print_exc()
-                else:
-                    b.connect('activate', self.__menu_activate, view)
-
-                    if (not min(list(
-                                map(self.__songinfo.can_change, b.needs))
-                                + [1])
-                            or comment.is_special()):
-                        b.set_sensitive(False)
-
-                    vals = b.activated(entry.tag, text)
-                    if len(vals) > 1 and vals[1][1]:
-                        split_menu.append(b)
-
+                item = self.__item_for(view, Item, entry.tag, text)
+                if not item:
+                    continue
+                vals = item.activated(entry.tag, text)
+                changeable = any(not self._group_info.can_change(k)
+                                 for k in item.needs)
+                fixed = changeable or comment.is_special()
+                if fixed:
+                    item.set_sensitive(False)
+                if len(vals) > 1 and vals[1][1]:
+                    split_menu.append(item)
             if split_menu.get_children():
                 split_menu.append(SeparatorMenuItem())
 
+            plugins = self.handler.plugins
+            print_d(f"Adding {len(plugins)} plugin(s) to menu: "
+                    f"{', '.join(p.__name__ for p in plugins)}")
+            for p_cls in plugins:
+                item = self.__item_for(view, p_cls, entry.tag, text)
+                if not item:
+                    continue
+                results = item.activated(entry.tag, text)
+                # Only enable for the user if the plugin would do something
+                item.set_sensitive(results != [(entry.tag, text)])
+                menu.append(item)
             pref_item = MenuItem(_("_Configure"), Icons.PREFERENCES_SYSTEM)
             split_menu.append(pref_item)
 
@@ -732,14 +764,14 @@ class EditTags(Gtk.VBox):
     def __add_new_tag(self, model, tag, value):
         assert isinstance(value, str)
         iters = [i for (i, v) in model.iterrows() if v.tag == tag]
-        if iters and not self.__songinfo.can_multiple_values(tag):
+        if iters and not self._group_info.can_multiple_values(tag):
             title = _("Unable to add tag")
-            msg = _("Unable to add <b>%s</b>") % util.escape(tag)
+            msg = _("Unable to add %s") % util.bold(tag)
             msg += "\n\n"
             msg += _("The files currently"
-                    " selected do not support multiple values for <b>%s</b>."
-                    ) % util.escape(tag)
-            qltk.ErrorMessage(self, title, msg).run()
+                     " selected do not support multiple values for %s."
+                     ) % util.bold(tag)
+            qltk.ErrorMessage(self, title, msg, escape_desc=False).run()
             return
 
         entry = ListEntry(tag, Comment(value))
@@ -751,7 +783,7 @@ class EditTags(Gtk.VBox):
             model.append(row=[entry])
 
     def __add_tag(self, activator, model, library):
-        add = AddTagDialog(self, self.__songinfo.can_change(), library)
+        add = AddTagDialog(self, self._group_info.can_change(), library)
 
         while True:
             resp = add.run()
@@ -762,11 +794,13 @@ class EditTags(Gtk.VBox):
             assert isinstance(value, str)
             value = massagers.validate(tag, value)
             assert isinstance(value, str)
-            if not self.__songinfo.can_change(tag):
-                title = _("Invalid tag")
-                msg = _("Invalid tag <b>%s</b>\n\nThe files currently"
-                        " selected do not support editing this tag."
-                        ) % util.escape(tag)
+            if not self._group_info.can_change(tag):
+                title = ngettext("Invalid tag", "Invalid tags", 1)
+                msg = ngettext("Invalid tag %s\n\nThe files currently "
+                        "selected do not support editing this tag.",
+                        "Invalid tags %s\n\nThe files currently "
+                        "selected do not support editing these tags.", 1
+                        ) % util.bold(tag)
                 qltk.ErrorMessage(self, title, msg).run()
             else:
                 self.__add_new_tag(model, tag, value)
@@ -825,7 +859,7 @@ class EditTags(Gtk.VBox):
                 l.append((entry.origtag, entry.value, entry.origvalue))
 
         was_changed = set()
-        songs = self.__songinfo.songs
+        songs = self._group_info.songs
         win = WritingWindow(self, len(songs))
         win.show()
         all_done = False
@@ -916,8 +950,8 @@ class EditTags(Gtk.VBox):
         if not massagers.is_valid(entry.tag, new_value):
             error_dialog = qltk.WarningMessage(
                 self, _("Invalid value"),
-                _("Invalid value: <b>%(value)s</b>\n\n%(error)s") % {
-                "value": new_value,
+                _("Invalid value: %(value)s\n\n%(error)s") % {
+                "value": util.bold(new_value),
                 "error": massagers.error_message(entry.tag, new_value)})
         else:
             new_value = massagers.validate(entry.tag, new_value)
@@ -942,13 +976,15 @@ class EditTags(Gtk.VBox):
         entry = model[path][0]
         if new_tag == entry.tag:
             return
-        elif not self.__songinfo.can_change(new_tag):
+        elif not self._group_info.can_change(new_tag):
             # Can't add the new tag.
-            title = _("Invalid tag")
-            msg = _("Invalid tag <b>%s</b>\n\nThe files currently"
-                    " selected do not support editing this tag."
-                    ) % util.escape(new_tag)
-            qltk.ErrorMessage(self, title, msg).run()
+            title = ngettext("Invalid tag", "Invalid tags", 1)
+            msg = ngettext("Invalid tag %s\n\nThe files currently "
+                    "selected do not support editing this tag.",
+                    "Invalid tags %s\n\nThe files currently "
+                    "selected do not support editing these tags.", 1
+                    ) % util.bold(new_tag)
+            qltk.ErrorMessage(self, title, msg, escape_desc=False).run()
         else:
             # FIXME: In case this is a special one we only
             # validate one value and never write it back..
@@ -957,8 +993,8 @@ class EditTags(Gtk.VBox):
             if not massagers.is_valid(new_tag, text):
                 qltk.WarningMessage(
                     self, _("Invalid value"),
-                    _("Invalid value: <b>%(value)s</b>\n\n%(error)s") % {
-                      "value": text,
+                    _("Invalid value: %(value)s\n\n%(error)s") % {
+                      "value": util.bold(text),
                       "error": massagers.error_message(new_tag, text)}).run()
                 return
             text = massagers.validate(new_tag, text)
@@ -1012,12 +1048,12 @@ class EditTags(Gtk.VBox):
 
     def _update(self, songs=None):
         if songs is None:
-            songs = self.__songinfo.songs
+            songs = self._group_info.songs
         else:
-            self.__songinfo = AudioFileGroup(songs)
-        songinfo = self.__songinfo
+            self._group_info = AudioFileGroup(songs)
+        info = self._group_info
 
-        keys = list(songinfo.keys())
+        keys = list(info.keys())
         default_tags = get_default_tags()
         keys = set(keys + default_tags)
 
@@ -1031,6 +1067,10 @@ class EditTags(Gtk.VBox):
         if not config.getboolean("editing", "alltags"):
             keys = filter(lambda k: k not in MACHINE_TAGS, keys)
 
+        if not config.getboolean("editing", "show_multi_line_tags"):
+            tags = config.getstringlist("editing", "multi_line_tags")
+            keys = filter(lambda k: k not in tags, keys)
+
         if not songs:
             keys = []
 
@@ -1038,16 +1078,16 @@ class EditTags(Gtk.VBox):
             model.clear()
 
             for tag in sorted(keys, key=custom_sort):
-                canedit = songinfo.can_change(tag)
+                canedit = info.can_change(tag)
 
                 # default tags
-                if tag not in songinfo:
+                if tag not in info:
                     entry = ListEntry(tag, Comment(u""))
                     entry.canedit = canedit
                     model.append(row=[entry])
                     continue
 
-                for value in songinfo[tag]:
+                for value in info[tag]:
                     entry = ListEntry(tag, value)
                     entry.origvalue = value
                     entry.edited = False
@@ -1057,7 +1097,7 @@ class EditTags(Gtk.VBox):
                     entry.origtag = ""
                     model.append(row=[entry])
 
-        self._buttonbox.set_sensitive(bool(songinfo.can_change()))
+        self._buttonbox.set_sensitive(bool(info.can_change()))
         self._revert.set_sensitive(False)
         self._remove.set_sensitive(False)
         self._save.set_sensitive(False)
@@ -1082,7 +1122,7 @@ class EditTags(Gtk.VBox):
 
     def __tag_editing_started(self, render, editable, path, model, library):
         if not editable.get_completion():
-            tags = self.__songinfo.can_change()
+            tags = self._group_info.can_change()
             if tags is True:
                 tags = USER_TAGS
             completion = qltk.EntryCompletion(tags)
