@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Copyright 2004-2005 Joe Wreschnig, Michael Urman, Iñigo Serna
 #
 # This program is free software; you can redistribute it and/or modify
@@ -13,7 +12,7 @@ from urllib.parse import urlsplit
 from gi.repository import Gtk, GObject, Gdk, Gio, Pango
 from senf import uri2fsn, fsnative, fsn2text, bytes2fsn
 
-from quodlibet import formats, print_d
+from quodlibet import formats, print_d, util
 from quodlibet import qltk
 from quodlibet import _
 
@@ -26,7 +25,7 @@ from quodlibet.qltk.x import ScrolledWindow, Paned
 from quodlibet.qltk.models import ObjectStore, ObjectTreeStore
 from quodlibet.qltk import Icons
 from quodlibet.util.path import listdir, \
-    glib2fsn, xdg_get_user_dirs, get_home_dir, xdg_get_config_home
+    xdg_get_user_dirs, get_home_dir, xdg_get_config_home
 from quodlibet.util import connect_obj
 
 
@@ -124,27 +123,17 @@ def get_favorites():
         return paths
 
 
-def _get_win_drives():
-    """Returns a list of paths for all available drives e.g. ['C:\\']"""
-
-    assert os.name == "nt"
-    drives = [letter + u":\\" for letter in u"CDEFGHIJKLMNOPQRSTUVWXYZ"]
-    return [d for d in drives if os.path.isdir(d)]
-
-
 def get_drives():
     """A list of accessible drives"""
 
-    if os.name == "nt":
-        return _get_win_drives()
-    else:
-        paths = []
-        for mount in Gio.VolumeMonitor.get().get_mounts():
-            path = mount.get_root().get_path()
-            if path is not None:
-                paths.append(glib2fsn(path))
+    paths = []
+    for mount in Gio.VolumeMonitor.get().get_mounts():
+        path = mount.get_root().get_path()
+        if path is not None:
+            paths.append(path)
+    if os.name != "nt":
         paths.append("/")
-        return paths
+    return sorted(paths)
 
 
 def parse_gtk_bookmarks(data):
@@ -202,7 +191,7 @@ class DirectoryTree(RCMHintedTreeView, MultiDragTreeView):
         """
 
         model = ObjectTreeStore()
-        super(DirectoryTree, self).__init__(model=model)
+        super().__init__(model=model)
 
         if initial is not None:
             assert isinstance(initial, fsnative)
@@ -296,7 +285,7 @@ class DirectoryTree(RCMHintedTreeView, MultiDragTreeView):
 
         # Find the top level row which has the largest common
         # path with the path we want to go to
-        roots = dict([(p, i) for (i, p) in model.iterrows(None)])
+        roots = dict((p, i) for (i, p) in model.iterrows(None))
         head, tail = path_to_go, fsnative(u"")
         to_find = []
         while head and head not in roots:
@@ -386,15 +375,14 @@ class DirectoryTree(RCMHintedTreeView, MultiDragTreeView):
         if not dir_:
             return
 
-        dir_ = glib2fsn(dir_)
         fullpath = os.path.realpath(os.path.join(directory, dir_))
 
         try:
             os.makedirs(fullpath)
         except EnvironmentError as err:
-            error = "<b>%s</b>: %s" % (err.filename, err.strerror)
+            error = f"{util.bold(err.filename)}: {util.escape(err.strerror)}"
             qltk.ErrorMessage(
-                None, _("Unable to create folder"), error).run()
+                None, _("Unable to create folder"), error, escape_desc=False).run()
             return
 
         self.emit('test-expand-row', model.get_iter(path), path)
@@ -409,9 +397,9 @@ class DirectoryTree(RCMHintedTreeView, MultiDragTreeView):
             try:
                 os.rmdir(directory)
             except EnvironmentError as err:
-                error = "<b>%s</b>: %s" % (err.filename, err.strerror)
+                error = f"{util.bold(err.filename)}: {err.strerror}"
                 qltk.ErrorMessage(
-                    None, _("Unable to delete folder"), error).run()
+                    None, _("Unable to delete folder"), error, escape_desc=False).run()
                 return
 
         ppath = Gtk.TreePath(paths[0][:-1])
@@ -512,7 +500,7 @@ class FileSelector(Paned):
         folders -- list of shown folders in the directory tree
         """
 
-        super(FileSelector, self).__init__(
+        super().__init__(
             orientation=Gtk.Orientation.VERTICAL)
         self.__filter = filter
 
@@ -681,7 +669,7 @@ class MainFileSelector(FileSelector):
 
     def __init__(self, initial=None):
         folders = _get_main_folders()
-        super(MainFileSelector, self).__init__(
+        super().__init__(
             initial, filesel_filter, folders=folders)
 
 
@@ -701,5 +689,5 @@ class MainDirectoryTree(DirectoryTree):
         if main:
             folders += main
 
-        super(MainDirectoryTree, self).__init__(
+        super().__init__(
             initial=initial, folders=folders)
