@@ -1,10 +1,10 @@
-# -*- coding: utf-8 -*-
 # Copyright 2004-2009 Joe Wreschnig, Michael Urman, Steven Robertson
-#           2011,2013 Nick Boultbee
+#           2011-2022 Nick Boultbee
 #
 # This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License version 2 as
-# published by the Free Software Foundation
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
 
 import os
 import random
@@ -12,40 +12,45 @@ import re
 import ctypes
 import ctypes.util
 import sys
-import traceback
 import unicodedata
 import threading
-import subprocess
-import webbrowser
-import contextlib
+import locale
+from typing import Dict, List, Callable
+from functools import reduce
 
 # Windows doesn't have fcntl, just don't lock for now
 try:
     import fcntl
     fcntl
 except ImportError:
-    fcntl = None
+    fcntl = None  # type: ignore
+
+from senf import fsnative
 
-from quodlibet.compat import reraise as py_reraise, urlparse
-from quodlibet.util.path import iscommand, is_fsnative
 from quodlibet.util.string.titlecase import title
 
 from quodlibet.const import SUPPORT_EMAIL, COPYRIGHT
-from quodlibet.util.dprint import print_d, print_
-from .misc import environ, argv, cached_func, get_locale_encoding, \
-    get_fs_encoding
-from .environment import *
+from quodlibet.util.dprint import print_d, print_, print_e, print_w, print_exc
+from .misc import cached_func, get_module_dir, get_ca_file, \
+    NamedTemporaryFile, cmp
+from .environment import is_plasma, is_unity, is_enlightenment, \
+    is_linux, is_windows, is_wine, is_osx, is_flatpak, matches_flatpak_runtime
+from .enum import enum
+from .i18n import _, ngettext, C_
 
 
-# pyflakes
-environ, argv, cached_func, get_locale_encoding, get_fs_encoding
+# flake8
+cached_func, enum, print_w, print_exc, is_plasma, is_unity, is_enlightenment,
+is_linux, is_windows, is_wine, is_osx, get_module_dir, get_ca_file,
+NamedTemporaryFile, is_flatpak, cmp, matches_flatpak_runtime
 
 
-class InstanceTracker(object):
+class InstanceTracker:
     """A mixin for GObjects to return a list of all alive objects
     of a given type. Note that it must be used with a GObject or
     something with a connect method and destroy signal."""
-    __kinds = {}
+
+    __kinds: Dict[type, List[object]] = {}
 
     def _register_instance(self, klass=None):
         """Register this object to be returned in the active instance list."""
@@ -59,7 +64,7 @@ class InstanceTracker(object):
         return klass.__kinds.get(klass, [])
 
 
-class OptionParser(object):
+class OptionParser:
     def __init__(self, name, version, description=None, usage=None):
         self.__name = name
         self.__version = version
@@ -73,7 +78,7 @@ class OptionParser(object):
             "help", shorts="h", help=_("Display brief usage information"))
         self.add(
             "version", shorts="v", help=_("Display version and copyright"))
-        self.add("debug", shorts="d")
+        self.add("debug", shorts="d", help=_("Print debugging information"))
 
     def add(self, canon, help=None, arg="", shorts="", longs=[]):
         self.__args[canon] = arg
@@ -92,10 +97,10 @@ class OptionParser(object):
 
     def __longs(self):
         longs = []
-        for long, arg in self.__args.items():
-            longs.append(long + (arg and "=" or ""))
-        for long, canon in self.__translate_long.items():
-            longs.append(long + (self.__args[canon] and "=" or ""))
+        for long_, arg in self.__args.items():
+            longs.append(long_ + (arg and "=" or ""))
+        for long_, canon in self.__translate_long.items():
+            longs.append(long_ + (self.__args[canon] and "=" or ""))
         return longs
 
     def __format_help(self, opt, space):
@@ -145,7 +150,7 @@ class OptionParser(object):
 {title} {version}
 <{email}>
 {copyright}\
-""").format(title=self.__name, version=self.__version, dates="2004-2012",
+""").format(title=self.__name, version=self.__version,
             email=SUPPORT_EMAIL, copyright=COPYRIGHT)
 
     def parse(self, args=None):
@@ -195,39 +200,47 @@ class OptionParser(object):
             return transopts, args
 
 
-def escape(str):
+def escape(string: str) -> str:
     """Escape a string in a manner suitable for XML/Pango."""
-    return str.replace("&", "&amp;").replace("<", "&lt;").replace(">", "&gt;")
+    return string.replace("&", "&amp;").replace("<", "&lt;").replace(">", "&gt;")
 
 
-def unescape(str):
+def unescape(string: str) -> str:
     """Unescape a string in a manner suitable for XML/Pango."""
-    return str.replace("&lt;", "<").replace("&gt;", ">").replace("&amp;", "&")
+    return string.replace("&lt;", "<").replace("&gt;", ">").replace("&amp;", "&")
 
 
-def bold(string):
-    return "<b>%s</b>" % string
+def bold(string: str, escaper: Callable[[str], str] = escape) -> str:
+    return "<b>%s</b>" % escaper(string)
 
 
-def monospace(string):
-    return "<tt>%s</tt>" % string
+def monospace(string: str, escaper: Callable[[str], str] = escape) -> str:
+    return "<tt>%s</tt>" % escaper(string)
 
 
-def italic(string):
-    return "<i>%s</i>" % string
+def italic(string: str, escaper: Callable[[str], str] = escape) -> str:
+    return "<i>%s</i>" % escaper(string)
 
 
-def parse_time(timestr, err=(ValueError, re.error)):
+def bold_italic(string: str, escaper: Callable[[str], str] = escape) -> str:
+    return "<b><i>%s</i></b>" % escaper(string)
+
+
+def parse_time(timestr, err=object()):
     """Parse a time string in hh:mm:ss, mm:ss, or ss format."""
+
     if timestr[0:1] == "-":
         m = -1
         timestr = timestr[1:]
     else:
         m = 1
+
     try:
         return m * reduce(lambda s, a: s * 60 + int(a),
                           re.split(r":|\.", timestr), 0)
-    except err:
+    except (ValueError, re.error):
+        if err is None:
+            raise
         return 0
 
 
@@ -303,7 +316,24 @@ def parse_date(datestr):
     except IndexError:
         raise ValueError
 
-    return time.mktime(time.strptime(datestr, frmt))
+    try:
+        return time.mktime(time.strptime(datestr, frmt))
+    except OverflowError as e:
+        raise ValueError(e)
+
+
+def format_int_locale(value):
+    """Turn an integer into a grouped, locale-dependent string
+    e.g. 12345 -> "12,345" or "12.345" etc
+    """
+    return locale.format_string("%d", value, grouping=True)
+
+
+def format_float_locale(value, format="%.2f"):
+    """Turn a float into a grouped, locale-dependent string
+    e.g. 12345.67 -> "12,345.67" or "12.345,67" etc
+    """
+    return locale.format_string(format, value, grouping=True)
 
 
 def format_rating(value, blank=True):
@@ -324,22 +354,28 @@ def format_bitrate(value):
 
 
 def format_size(size):
-    """Turn an integer size value into something human-readable."""
+    """Turn an integer size value into something human-readable.
+
+    Args:
+        size (int): size in bytes
+    Returns:
+        str
+    """
     # TODO: Better i18n of this (eg use O/KO/MO/GO in French)
     if size >= 1024 ** 3:
-        return "%.1f GB" % (float(size) / (1024 ** 3))
+        return u"%.1f GB" % (float(size) / (1024 ** 3))
     elif size >= 1024 ** 2 * 100:
-        return "%.0f MB" % (float(size) / (1024 ** 2))
+        return u"%.0f MB" % (float(size) / (1024 ** 2))
     elif size >= 1024 ** 2 * 10:
-        return "%.1f MB" % (float(size) / (1024 ** 2))
+        return u"%.1f MB" % (float(size) / (1024 ** 2))
     elif size >= 1024 ** 2:
-        return "%.2f MB" % (float(size) / (1024 ** 2))
+        return u"%.2f MB" % (float(size) / (1024 ** 2))
     elif size >= 1024 * 10:
-        return "%d KB" % int(size / 1024)
+        return u"%d KB" % int(size / 1024)
     elif size >= 1024:
-        return "%.2f KB" % (float(size) / 1024)
+        return u"%.2f KB" % (float(size) / 1024)
     else:
-        return "%d B" % size
+        return u"%d B" % size
 
 
 def format_time(time):
@@ -365,6 +401,13 @@ def format_time_display(time):
     return format_time(time).replace(":", u"\u2236")
 
 
+def format_time_seconds(time):
+    from quodlibet import ngettext
+
+    time_str = format_int_locale(time)
+    return ngettext("%s second", "%s seconds", time) % time_str
+
+
 def format_time_long(time, limit=2):
     """Turn a time value in seconds into x hours, x minutes, etc.
 
@@ -372,6 +415,8 @@ def format_time_long(time, limit=2):
     0 means no limit.
     """
 
+    from quodlibet import ngettext
+
     if time < 1:
         return _("No time information")
 
@@ -401,6 +446,21 @@ def format_time_long(time, limit=2):
     return ", ".join(time_str)
 
 
+def format_time_preferred(t, fmt=None):
+    """Returns duration formatted to user's preference"""
+    from quodlibet.config import DurationFormat, DURATION
+    fmt = fmt or DURATION.format
+
+    if fmt == DurationFormat.STANDARD:
+        return format_time_long(t, 2)
+    elif fmt == DurationFormat.NUMERIC:
+        return format_time_display(t)
+    elif fmt == DurationFormat.FULL:
+        return format_time_long(t, 5)
+    else:
+        return format_time_seconds(t)
+
+
 def capitalize(str):
     """Capitalize a string, not affecting any character after the first."""
     return str[:1].upper() + str[1:]
@@ -413,7 +473,8 @@ def _split_numeric_sortkey(s, limit=10,
     it can be used for human sorting. Also removes all extra whitespace."""
     result = reg(s)
     if not result or not limit:
-        return (join(s.split()),)
+        text = join(s.split())
+        return (text,) if text else tuple()
     else:
         start, end = result.span()
         return (
@@ -423,53 +484,23 @@ def _split_numeric_sortkey(s, limit=10,
 
 
 def human_sort_key(s, normalize=unicodedata.normalize):
-    if not isinstance(s, unicode):
+    if not s:
+        return ()
+    if not isinstance(s, str):
         s = s.decode("utf-8")
     s = normalize("NFD", s.lower())
-    return s and _split_numeric_sortkey(s)
+    return _split_numeric_sortkey(s)
 
 
 def website(site):
     """Open the given URL in the user's default browser"""
 
-    if os.name == "nt" or sys.platform == "darwin":
-        return webbrowser.open(site)
+    from gi.repository import Gtk, Gdk, GLib
 
-    # all commands here return immediately
-    for prog in ["xdg-open", "gnome-open"]:
-        if not iscommand(prog):
-            continue
-
-        status = subprocess.check_call([prog, site])
-        if status == 0:
-            return True
-
-    # sensible-browser is a debian thing
-    blocking_progs = ["sensible-browser"]
-    blocking_progs.extend(environ.get("BROWSER", "").split(":"))
-
-    for prog in blocking_progs:
-        if not iscommand(prog):
-            continue
-
-        # replace %s with the url
-        args = prog.split()
-        for i, arg in enumerate(args):
-            if arg == "%s":
-                args[i] = site
-                break
-        else:
-            args.append(site)
-
-        # calling e.g. firefox blocks, so call async and hope for the best
-        try:
-            spawn(args)
-        except RuntimeError:
-            continue
-        else:
-            return True
-
-    return False
+    try:
+        Gtk.show_uri(None, site, Gdk.CURRENT_TIME)
+    except GLib.Error:
+        print_exc()
 
 
 def tag(name, cap=True):
@@ -477,7 +508,7 @@ def tag(name, cap=True):
     # Strips ~ and ~# from the start and runs it through a map (which
     # the user can configure).
     if not name:
-        return _("Invalid tag")
+        return ngettext("Invalid tag", "Invalid tags", 1)
     else:
         from quodlibet.util.tags import readable
         parts = map(readable, tagsplit(name))
@@ -485,7 +516,7 @@ def tag(name, cap=True):
             # Translators: If tag names, when capitalized, should not
             # be title-cased ("Looks Like This"), but rather only have
             # the first letter capitalized, translate this string as
-            # something other than "titlecase?".
+            # something non-empty other than "titlecase?".
             if C_("check", "titlecase?") == "titlecase?":
                 parts = map(title, parts)
             else:
@@ -512,10 +543,13 @@ def tagsplit(tag):
         return [tag]
 
 
-def pattern(pat, cap=True, esc=False):
+def pattern(pat, cap=True, esc=False, markup=False):
     """Return a 'natural' version of the pattern string for human-readable
-    bits. Assumes all tags in the pattern are present."""
-    from quodlibet.pattern import Pattern, XMLFromPattern
+    bits. Assumes all tags in the pattern are present.
+    """
+
+    from quodlibet.pattern import Pattern, XMLFromPattern, XMLFromMarkupPattern
+    from quodlibet.formats import FILESYSTEM_TAGS
 
     class Fakesong(dict):
         cap = False
@@ -527,13 +561,20 @@ def pattern(pat, cap=True, esc=False):
             return [tag(k, self.cap) for k in tagsplit(key)]
         list_separate = list
 
-        def __call__(self, tag, default):
+        def __call__(self, tag, *args):
+            if tag in FILESYSTEM_TAGS:
+                return fsnative(str(tag))
             return 0 if '~#' in tag[:2] else self.comma(tag)
 
     fakesong = Fakesong({'filename': tag('filename', cap)})
     fakesong.cap = cap
     try:
-        p = (esc and XMLFromPattern(pat)) or Pattern(pat)
+        if markup:
+            p = XMLFromMarkupPattern(pat)
+        elif esc:
+            p = XMLFromPattern(pat)
+        else:
+            p = Pattern(pat)
     except ValueError:
         return _("Invalid pattern")
 
@@ -551,10 +592,7 @@ def spawn(argv, stdout=False):
 
     from gi.repository import GLib
 
-    types = map(type, argv)
-    if not (min(types) == max(types) == str):
-        raise TypeError("executables and arguments must be str objects")
-    print_d("Running %r" % " ".join(argv))
+    print_d("Running %r" % argv)
     args = GLib.spawn_async(argv=argv, flags=GLib.SpawnFlags.SEARCH_PATH,
                             standard_output=stdout)
 
@@ -568,22 +606,11 @@ def fver(tup):
     return ".".join(map(str, tup))
 
 
-def uri_is_valid(uri):
-    return bool(urlparse(uri)[0])
-
-
 def make_case_insensitive(filename):
     return "".join(["[%s%s]" % (c.lower(), c.upper()) for c in filename])
 
 
-def print_exc(limit=None, file=None):
-    """A wrapper preventing crashes on broken pipes in print_exc."""
-    if not file:
-        file = sys.stderr
-    print_(traceback.format_exc(limit=limit), output=file)
-
-
-class DeferredSignal(object):
+class DeferredSignal:
     """A wrapper for connecting functions to signals.
 
     Some signals may fire hundreds of times, but only require processing
@@ -591,7 +618,7 @@ class DeferredSignal(object):
     priority and prevents multiple calls from being inserted in the
     mainloop at a time, greatly improving responsiveness in some places.
 
-    When the target function will finally be called the arguments passed
+    When the target function is finally called, the arguments passed
     are the last arguments passed to DeferredSignal.
 
     `priority` defaults to GLib.PRIORITY_DEFAULT
@@ -630,8 +657,8 @@ class DeferredSignal(object):
                 timeout, f, priority=priority)
 
     @property
-    def im_self(self):
-        return self.func.im_self
+    def __self__(self):
+        return self.func.__self__
 
     @property
     def __code__(self):
@@ -641,6 +668,13 @@ class DeferredSignal(object):
     def __closure__(self):
         return self.func.__closure__
 
+    def call(self, *args):
+        """Force a call"""
+
+        self.abort()
+        self.args = args
+        self._wrap()
+
     def abort(self):
         """Abort any queued up calls.
 
@@ -695,8 +729,8 @@ def _connect_destroy(sender, func, detailed_signal, handler, *args, **kwargs):
     to the bound method and the bound to object doesn't get GCed.
     """
 
-    if hasattr(handler, "im_self"):
-        obj = handler.im_self
+    if hasattr(handler, "__self__"):
+        obj = handler.__self__
     else:
         # XXX: get the "self" var of the enclosing scope.
         # Used for nested functions which ref the object but aren't methods.
@@ -723,14 +757,15 @@ def connect_after_destroy(sender, *args, **kwargs):
     return _connect_destroy(sender, sender.connect_after, *args, **kwargs)
 
 
-class cached_property(object):
-    """A read-only @property that is only evaluated once."""
+class cached_property:
+    """A read-only @property that is only evaluated once.
+    TODO: work out some typing for this, see test_query.py"""
 
     def __init__(self, fget, doc=None):
         self.fget = fget
         self.__doc__ = doc or fget.__doc__
         self.__name__ = name = fget.__name__
-        # these get name mangled, so caching wont work unless
+        # these get name mangled, so caching won't work unless
         # we mangle too
         assert not (name.startswith("__") and not name.endswith("__")), \
             "can't cache a dunder method"
@@ -749,11 +784,11 @@ def sanitize_tags(tags, stream=False):
     """
 
     san = {}
-    for key, value in tags.iteritems():
+    for key, value in tags.items():
         key = key.lower()
         key = {"location": "website"}.get(key, key)
 
-        if isinstance(value, unicode):
+        if isinstance(value, str):
             lower = value.lower().strip()
 
             if key == "channel-mode":
@@ -774,7 +809,7 @@ def sanitize_tags(tags, stream=False):
 
         if key == "duration":
             try:
-                value = int(long(value) / 1000)
+                value = int(int(value) / 1000)
             except ValueError:
                 pass
             else:
@@ -807,7 +842,7 @@ def sanitize_tags(tags, stream=False):
         if not stream and key in ("title", "album", "artist", "date"):
             continue
 
-        if isinstance(value, (int, long, float)):
+        if isinstance(value, (int, float)):
             if not key.startswith("~#"):
                 key = "~#" + key
             san[key] = value
@@ -815,7 +850,7 @@ def sanitize_tags(tags, stream=False):
             if key.startswith("~#"):
                 key = key[2:]
 
-            if not isinstance(value, unicode):
+            if not isinstance(value, str):
                 continue
 
             value = value.strip()
@@ -864,15 +899,10 @@ def limit_songs(songs, max, weight_by_ratings=False):
         return songs
     else:
         if weight_by_ratings:
-            def choose(r1, r2):
-                if r1 or r2:
-                    return cmp(random.random(), r1 / (r1 + r2))
-                else:
-                    return random.randint(-1, 1)
-
-            def rating(song):
-                return song("~#rating")
-            songs.sort(cmp=choose, key=rating)
+            def rating_weighted_random(song):
+                # Apply even (random : higher rating) weighting
+                return (1 - song("~#rating")) * (1 + random.random())
+            songs.sort(key=rating_weighted_random)
         else:
             random.shuffle(songs)
         return songs[:max]
@@ -888,56 +918,16 @@ def gi_require_versions(name, versions):
 
     import gi
 
+    error = None
     for version in versions:
         try:
             gi.require_version(name, version)
         except ValueError as e:
-            pass
+            error = e
         else:
             return version
     else:
-        raise e
-
-
-@contextlib.contextmanager
-def atomic_save(filename, suffix, mode):
-    """Try to replace the content of a file in the safest way possible.
-
-    * filename+suffix will be created during the process.
-    * On UNIX this operation is atomic, on Windows it is not.
-
-    with atomic_save("config.cfg", ".tmp", "wb") as f:
-        f.write(data)
-
-    Can raise.
-    """
-
-    assert is_fsnative(filename)
-
-    temp_filename = filename + suffix
-    fileobj = open(temp_filename, "wb")
-    try:
-        if fcntl is not None:
-            fcntl.flock(fileobj.fileno(), fcntl.LOCK_EX)
-
-        yield fileobj
-
-        fileobj.flush()
-        os.fsync(fileobj.fileno())
-
-        # No atomic rename on windows
-        if os.name == "nt":
-            fileobj.close()
-            try:
-                os.remove(filename)
-            except EnvironmentError:
-                pass
-
-        os.rename(temp_filename, filename)
-    finally:
-        if fcntl is not None:
-            fcntl.flock(fileobj.fileno(), fcntl.LOCK_UN)
-        fileobj.close()
+        raise error
 
 
 def load_library(names, shared=True):
@@ -959,18 +949,6 @@ def load_library(names, shared=True):
     else:
         load_func = ctypes.cdll.LoadLibrary
 
-    if is_osx():
-        # make sure it's either empty or contains /usr/lib.
-        # (jhbuild sets it for example). Otherwise ctypes can't
-        # find libc (bug?)
-        if "DYLD_FALLBACK_LIBRARY_PATH" in os.environ:
-            paths = os.environ["DYLD_FALLBACK_LIBRARY_PATH"]
-            paths = paths.split(os.pathsep)
-            if "/usr/lib" not in paths:
-                paths.append("/usr/lib")
-                os.environ["DYLD_FALLBACK_LIBRARY_PATH"] = \
-                    os.pathsep.join(paths)
-
     errors = []
     for name in names:
         dlopen_name = name
@@ -978,6 +956,9 @@ def load_library(names, shared=True):
                 ".dylib" not in name:
             dlopen_name = ctypes.util.find_library(name) or name
 
+        if is_osx() and not os.path.isabs(dlopen_name):
+            dlopen_name = os.path.join(sys.prefix, "lib", dlopen_name)
+
         try:
             return load_func(dlopen_name), name
         except OSError as e:
@@ -1004,7 +985,7 @@ class MainRunnerTimeoutError(MainRunnerError):
     pass
 
 
-class MainRunner(object):
+class MainRunner:
     """Schedule a function call in the main loop from a
     worker thread and wait for the result.
 
@@ -1065,7 +1046,7 @@ class MainRunner(object):
 
     def call(self, func, *args, **kwargs):
         """Runs the function in the main loop and blocks until
-        it is finshed or abort() was called. In case this is called
+        it is finished or abort() was called. In case this is called
         from the main loop the function gets executed immediately.
 
         The priority kwargs defines the event source priority and will
@@ -1114,47 +1095,12 @@ class MainRunner(object):
 
 
 def re_escape(string, BAD="/.^$*+-?{,\\[]|()<>#=!:"):
-    """A re.escape which also works with unicode"""
+    """A `re.escape` which also works with unicode"""
 
     needs_escape = lambda c: (c in BAD and "\\" + c) or c
     return type(string)().join(map(needs_escape, string))
 
 
-def enum(cls):
-    """Class decorator for enum types::
-
-        @enum
-        class SomeEnum(int):
-            FOO = 0
-            BAR = 1
-
-    Result is an int subclass and all attributes are instances of it.
-    """
-
-    type_ = cls.__bases__[0]
-
-    d = dict(cls.__dict__)
-    new_type = type(cls.__name__, (type_,), d)
-    new_type.__module__ = cls.__module__
-
-    map_ = {}
-    for key, value in d.iteritems():
-        if key.upper() == key and isinstance(value, type_):
-            value_instance = new_type(value)
-            setattr(new_type, key, value_instance)
-            map_[value] = key
-
-    def repr_(self):
-        if self in map_:
-            return "%s.%s" % (type(self).__name__, map_[self])
-        else:
-            return "%s(%s)" % (type(self).__name__, self)
-
-    setattr(new_type, "__repr__", repr_)
-
-    return new_type
-
-
 def set_process_title(title):
     """Sets process name as visible in ps or top. Requires ctypes libc
     and is almost certainly *nix-only. See issue 736
@@ -1165,11 +1111,22 @@ def set_process_title(title):
 
     try:
         libc = load_library(["libc.so.6", "c"])[0]
-        # 15 = PR_SET_NAME, apparently
-        libc.prctl(15, title, 0, 0, 0)
+        prctl = libc.prctl
     except (OSError, AttributeError):
         print_d("Couldn't find module libc.so.6 (ctypes). "
                 "Not setting process title.")
+    else:
+        prctl.argtypes = [
+            ctypes.c_int, ctypes.c_ulong, ctypes.c_ulong,
+            ctypes.c_ulong, ctypes.c_ulong,
+        ]
+        prctl.restype = ctypes.c_int
+
+        PR_SET_NAME = 15
+        data = ctypes.create_string_buffer(title.encode("utf-8"))
+        res = prctl(PR_SET_NAME, ctypes.addressof(data), 0, 0, 0)
+        if res != 0:
+            print_w("Setting the process title failed")
 
 
 def list_unique(sequence):
@@ -1195,4 +1152,4 @@ def reraise(tp, value, tb=None):
 
     if tb is None:
         tb = sys.exc_info()[2]
-    py_reraise(tp, value, tb)
+    raise tp(value).with_traceback(tb)
