@@ -1,17 +1,15 @@
-# -*- coding: utf-8 -*-
 # Copyright 2012, 2013 Christoph Reiter
 #
 # This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License version 2 as
-# published by the Free Software Foundation
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
 
 import xml.etree.ElementTree as ET
 
 import dbus
 import dbus.service
 
-from quodlibet.compat import unichr, text_type, PY3, iteritems
-
 
 def dbus_unicode_validate(text):
     """Takes a unicode string and replaces all invalid codepoints that would
@@ -33,7 +31,7 @@ def dbus_unicode_validate(text):
             cps.append(c)
         else:
             cps.append(0xFFFD)
-    return u"".join(map(unichr, cps))
+    return u"".join(map(chr, cps))
 
 
 def list_spec_properties(spec):
@@ -46,9 +44,7 @@ def list_spec_properties(spec):
     'emit' can be true/false/invalidates (see dbus spec)
     """
 
-    if not isinstance(spec, bytes):
-        spec = spec.encode("utf-8")
-    assert isinstance(spec, bytes)
+    assert isinstance(spec, str)
 
     ANNOTATION_EMITS = "org.freedesktop.DBus.Property.EmitsChangedSignal"
 
@@ -61,7 +57,10 @@ def list_spec_properties(spec):
             emit = fallback
         return emit
 
-    root = ET.fromstring(b'<?xml version="1.0"?><props>' + spec + b'</props>')
+    root = ET.fromstring(
+        b'<?xml version="1.0" encoding="UTF-8"?><props>' +
+        spec.encode("utf-8") +
+        b'</props>')
     props = {}
     root_emit = get_emit(root, "true")
     for element in root:
@@ -79,15 +78,17 @@ def list_spec_properties(spec):
 def filter_property_spec(spec, wl=None, bl=None):
     """Remove properties based on a white list or black list."""
 
-    if not isinstance(spec, bytes):
-        spec = spec.encode("utf-8")
-    assert isinstance(spec, bytes)
+    assert isinstance(spec, str)
 
     if wl and bl:
         raise ValueError
     if not wl and not bl:
         return spec
-    root = ET.fromstring(b'<?xml version="1.0"?><props>' + spec + b'</props>')
+
+    root = ET.fromstring(
+        b'<?xml version="1.0" encoding="UTF-8"?><props>' +
+        spec.encode("utf-8") +
+        b'</props>')
     if wl:
         to_rm = lambda e: e.attrib["name"] not in wl
     elif bl:
@@ -95,8 +96,8 @@ def filter_property_spec(spec, wl=None, bl=None):
     strs = []
     for element in root:
         if element.tag != "property" or not to_rm(element):
-            strs.append(ET.tostring(element).strip())
-    return b"\n".join(strs)
+            strs.append(ET.tostring(element, encoding="unicode").strip())
+    return "\n".join(strs)
 
 
 TYPE_MAP = {
@@ -131,14 +132,9 @@ def apply_signature(value, sig, utf8_strings=False):
     elif sig.startswith("a"):
         return dbus.Array(value, signature=sig[1:])
     elif sig == "s":
-        if utf8_strings and not PY3:
-            if isinstance(value, text_type):
-                value = value.encode("utf-8")
-            return dbus.UTF8String(value)
-        else:
-            if isinstance(value, bytes):
-                value = value.decode("utf-8")
-            return dbus.String(value)
+        if isinstance(value, bytes):
+            value = value.decode("utf-8")
+        return dbus.String(value)
     else:
         return TYPE_MAP[sig](value)
 
@@ -146,7 +142,7 @@ def apply_signature(value, sig, utf8_strings=False):
     return value
 
 
-class DBusIntrospectable(object):
+class DBusIntrospectable:
     """Simply collects all introspection data from other mixins
     and provides the Introspect DBus method returning all combined.
 
@@ -154,7 +150,7 @@ class DBusIntrospectable(object):
     and provided signals, properties, methods in the introspection
     xml format.
 
-    The dbus bindings allready provide a Introspect method, but it doesn't
+    The dbus bindings already provide a Introspect method, but it doesn't
     understand properties, also having them in text format in the class
     is a nice documentation.
     """
@@ -178,15 +174,15 @@ class DBusIntrospectable(object):
     def Introspect(self):
         parts = []
         parts.append("<node>")
-        for iface, intros in iteritems(self.__ispec):
+        for iface, intros in self.__ispec.items():
             parts.append("<interface name=\"%s\">" % iface)
             parts.extend(intros)
             parts.append("</interface>")
         parts.append("</node>")
-        return ("\n".join(parts)).encode("utf-8")
+        return "\n".join(parts)
 
 
-class DBusProperty(object):
+class DBusProperty:
     """A mixin for dbus.Object classes to support dbus properties.
 
     Register properties by passing the XML introspection to
@@ -195,7 +191,7 @@ class DBusProperty(object):
     The class needs to provide 'get/set_property'.
 
     In case the base Object is a FallbackObject, 'get/set_property' also
-    need to handle an additional realtive path parameter.
+    need to handle an additional relative path parameter.
 
     Whenever a property changes, 'emit_properties_changed' needs to be
     called (except if the annotations disable it). In case of
@@ -289,7 +285,7 @@ class DBusProperty(object):
                 raise ValueError("Property %s not registered" % prop)
             combos.setdefault(iface, []).append(prop)
 
-        for iface, props in iteritems(combos):
+        for iface, props in combos.items():
             values = {}
             inval = []
             for prop in props:
