@@ -1,4 +1,5 @@
 # Copyright 2017 Christoph Reiter
+#           2021 halfbrained@github
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -14,8 +15,21 @@ from tests.plugin import PluginTestCase
 AUDIO_FILE = SongWrapper(AudioFile({'~filename': "/tmp/foobar"}))
 
 
-class TConsole(PluginTestCase):
+class Dummy:
+    dummy_val = "str!"
 
+    def dummy_meth(self, arg, varg=101):
+        pass
+
+
+DUMMY_COMPLETIONS = [
+    ('dummy_meth', ' (arg, varg=101)'),
+    ('dummy_val', ''),
+]
+NAMESPACE_COMPLETIONS = ('dummy', '')
+
+
+class TConsole(PluginTestCase):
     def setUp(self):
         self.mod = self.modules["Python Console Sidebar"]
 
@@ -30,3 +44,17 @@ class TConsole(PluginTestCase):
         self.failUnlessEqual(plugin.console.namespace.get('songs'),
                              [AUDIO_FILE])
         plugin.disabled()
+
+    def test_console_completion(self):
+        plugin = self.mod.PyConsoleSidebar()
+        plugin.enabled()
+
+        plugin.console.namespace['dummy'] = Dummy()
+
+        comp = plugin.console.get_completion_items('dummy.')
+        self.failUnlessEqual(comp, DUMMY_COMPLETIONS)
+
+        comp = plugin.console.get_completion_items('')
+        assert NAMESPACE_COMPLETIONS in comp
+
+        plugin.disabled()
