@@ -23,7 +23,7 @@ except ImportError:
     pass
 
 from quodlibet.player import PlayerError
-from quodlibet.util import sanitize_tags, is_flatpak, matches_flatpak_runtime
+from quodlibet.util import sanitize_tags, is_flatpak, matches_flatpak_runtime, is_osx
 from quodlibet.formats import MusicFile
 from quodlibet import config
 
@@ -246,7 +246,6 @@ class TGStreamerCodecs(TestCase):
             "empty.ogg",
             "empty.opus",
             "silence-44-s.mpc",
-            "silence-44-s.sv8.mpc",
             "silence-44-s.tta",
             # "test.mid",
             "test.spc",
@@ -261,6 +260,10 @@ class TGStreamerCodecs(TestCase):
             # https://gitlab.com/freedesktop-sdk/freedesktop-sdk/issues/809
             files.append("silence-44-s.spx")
 
+        if not is_osx():
+            # gstlibav gets stuck decoding this.. not sure
+            files.append("silence-44-s.sv8.mpc")
+
         errors = []
         for file_ in files:
             path = get_data_path(file_)
