@@ -5,13 +5,14 @@
 
 import pytest
 
-from tests import TestCase
+from tests import TestCase, run_gtk_loop
 
 from gi.repository import Gtk
 
 from quodlibet.util import copool
 
 
+@pytest.mark.flaky(max_runs=3, min_passes=2)
 class Tcopool(TestCase):
     def setUp(self):
         self.buffer = None
@@ -26,15 +27,14 @@ class Tcopool(TestCase):
             yield None
 
     def _assert_eventually(self, value):
-        for i in range(100):
+        while Gtk.events_pending():
+            Gtk.main_iteration()
             if self.buffer is value:
                 return
-            Gtk.main_iteration_do(False)
         assert self.buffer is value
 
     def _assert_never(self, value):
-        for i in range(100):
-            Gtk.main_iteration_do(False)
+        run_gtk_loop()
         assert self.buffer is not value
 
     def test_add_remove(self):
