@@ -7,7 +7,7 @@ import os
 import shutil
 import unittest
 
-from senf import uri2fsn, fsn2uri, fsnative, environ
+from senf import uri2fsn, fsn2uri, fsnative
 
 from quodlibet.util.path import iscommand, limit_path, \
     get_home_dir, uri_is_valid, is_hidden, uri2gsturi
@@ -16,7 +16,7 @@ from quodlibet.util import print_d
 from . import TestCase, skipIf
 
 is_win = os.name == "nt"
-path_set = bool(environ.get('PATH', False))
+path_set = bool(os.environ.get('PATH', False))
 
 
 def test_uri2gsturi():
@@ -155,7 +155,7 @@ class Tiscommand(TestCase):
     @unittest.skipUnless(path_set, "Can only test with a valid $PATH")
     @unittest.skipIf(is_win, "needs porting")
     def test_looks_in_path(self):
-        path_dirs = set(environ['PATH'].split(os.path.pathsep))
+        path_dirs = set(os.environ['PATH'].split(os.path.pathsep))
         dirs = path_dirs - set(os.defpath.split(os.path.pathsep))
         for d in dirs:
             if os.path.isdir(d):
